/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot;

import java.util.ArrayList;


/**
 * Data that is stored in memory in a list of a particular type
 */
public abstract class MemoryListData<T> extends ArrayList<T> implements Data<T>  {

	public void addAll(MemoryListData<T> t) {
		Iterable<T> list = t.get();
		addAll(list);
	}

	public void addAll(Iterable<T> list) {
		for (T t : list)
			this.add(t);
	}



	

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.Data#get(int, int)
	 */
	@Override
	public Iterable<T> get(int start, int end) {
		return new LocatedListIterable(this, new Location(start,end));
		
				
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.Data#get()
	 */
	@Override
	public Iterable<T> get() {
		return new ListIterable<T>(this);
	}

	/* (non-Javadoc)
	 * @see net.sf.jannot.Data#canSave()
	 */
	@Override
	public boolean canSave() {
		return false;
	}

//	/**
//	 * @return
//	 */
//	public int size() {
//		return list.size();
//	}
//
//	/**
//	 * @param row
//	 * @return
//	 */
//	public T get(int row) {
//		return list.get(row);
//	}
//
//	/**
//	 * @param first
//	 * @return
//	 */
//	public int indexOf(T o) {
//		return list.indexOf(o);
//	}

}
