/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.parser.software;

import java.io.InputStream;

import net.sf.jannot.EntrySet;
import net.sf.jannot.Feature;
import net.sf.jannot.Location;
import net.sf.jannot.MemoryFeatureAnnotation;
import net.sf.jannot.Strand;
import net.sf.jannot.Type;
import net.sf.jannot.parser.Parser;
import be.abeel.io.LineIterator;

/**
 * Parser for the output of the GeneMark gene caller
 * 
 * @author Thomas Abeel
 * 
 */
public class GeneMarkParser extends Parser {

	/**
	 * @param dataKey
	 */
	public GeneMarkParser() {
		super(null);
	}

	@Override
	public EntrySet parse(InputStream is, EntrySet set) {
		if(set==null)
    		set=new EntrySet();
		Type t=Type.get("CDS_pred");
		MemoryFeatureAnnotation fa=set.iterator().next().getMemoryAnnotation(t);
		for(String line:new LineIterator(is)){
			if(!line.startsWith(" ")||line.contains("Gene")||line.contains("Length"))
				continue;
			String[]arr=line.replace('<', ' ').trim().split("[ \t]+");
			Feature f=new Feature();
			f.addLocation(new Location(Integer.parseInt(arr[2]),Integer.parseInt(arr[3])));
			f.setStrand(Strand.fromSymbol(arr[1].charAt(0)));
			f.setType(t);
			f.addQualifier("source", "GeneMark");
			f.addQualifier("Gene",arr[0]);
			fa.add(f);
		}
		
		
		return set;
	}

}
