/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.shortread;

import net.sf.jannot.Strand;
/**
 * 
 * @author Thomas Abeel
 *
 */
public abstract class ShortRead implements Comparable<ShortRead> {

	public static ShortRead getQuery(int pos) {
		return new BasicShortRead(new byte[0], pos, true);
	}

	/*
	 * These should be one-based coordinates
	 */
	public abstract char getNucleotide(int pos);

	@Override
	public int compareTo(ShortRead o) {
		if (this == o)
			return 0;
		int comp = new Integer(this.start()).compareTo(new Integer(o.start()));
		if (comp == 0)
			comp++;
		return comp;
	}

	@Override
	public boolean equals(Object obj) {
		return this == obj;
	}

	public abstract int start();

	public abstract int end();

	public abstract Strand strand();

	public abstract int length();

	// @Override
	// public String toString() {
	//
	// }

	public boolean isFirstInPair() {
		return true;
	}

	

	public boolean isPaired() {
		return false;
	}
}
